Option Explicit

' Serial communication example: midi

' outputBuffer will store the bytes we wish to send out to the world:
dim outputBuffer(1 to 15) 	as byte
' inputBuffer will receive bytes from the outside world
dim inputBuffer(1 to 10) 	as byte

' variables and constants for the pot
dim potVar1 as integer
const potPin1 as byte = 13

dim potVar2 as integer
const potPin2 as byte = 14

' set up midi data variables:
dim status 	as byte
dim data1 	as byte
dim data2 	as byte
dim count		as byte
dim pauseTime as single

' declare data that will be read from text files
private midiNotes 		as New ByteVectorDataRW
private midiVelocities as New ByteVectorDataRW
private midiDurations 	as New ByteVectorDataRW

' stack memory for PB task
dim vibratoTaskStack(1 to 200) as byte

Public Sub Main()

	' light red LED: starting up
	putpin 25, 0
	putpin 26, 1

	delay 0.5

	' set up serial communication
	' using com1 for midi data out
	call openQueue(inputBuffer, 10)
	call openQueue(outputBuffer, 15)
	call openCom(1, 9600, inputBuffer, outputBuffer)
	register.ubrr = 14	


	' load data from text file
	call midiNotes.Source("midiNotes1.txt")
	call midiVelocities.Source("midiVelocities1.txt")
	call midiDurations.Source("midiDurations1.txt")

	callTask "vibratoTask", vibratoTaskStack

	delay 0.25
	' switch to green led: everything started:
	putpin 25, 1
	putpin 26, 0
	
	status 	= 144 	' noteOn, channel 1
	count 	= 1

	do
		' read values from vectors
		data1 = cByte(midiNotes(count))
		data2 = cByte(midiVelocities(count))

		'read the pot
		potVar1 = getADC(potPin1)
		if potVar1 < 5 then
			potVar1 = 5
		end if
		pauseTime = cSng(midiDurations(count))/(cSng(potVar1 + 1)/64.0)

		' put midi data in output queue:
		call putQueue(outputBuffer, status, 1)
		call putQueue(outputBuffer, data1, 1)
		call putQueue(outputBuffer, data2, 1)

		' flash green led:
		putpin 26, 1
		delay pauseTime / 2.0
		putpin 26, 0
		delay pauseTime / 2.0

		count = count + 1
		if (count > 15) then
			count = 1
		end if

	loop

End Sub



sub vibratoTask()
	dim PBStatus as byte
	dim upper as byte
	dim lower as byte

	dim counter as single
	dim pb as integer
	dim temp as integer
	dim delta as single

	counter = 1.0
	delta = 0.1
	PBStatus = 224
	do
		' convert the sine value (-1 to 1) to a number between 0 and 2^14
		pb = cInt(((sin(counter)+1.0) / 2.0) * (16384.0))
		
		' to get the upper 7 bits, shift the integer value right 7 places,
		' then perform a bitwise AND with a mask that only passes the bottom 7 bits
		upper = cByte(cByte(pb\128) AND bx0111_1111)
		' to get the lower 7 bites, erase the upper bits by sliding the whole number left 8 bits and back
		temp = pb * 256
		' then do the same bitwise AND
		lower = cByte(cByte(temp\256) AND bx0111_1111)

		' send the midi data out
		call putQueue(outputBuffer, PBStatus, 1)
		call putQueue(outputBuffer, lower, 1)
		call putQueue(outputBuffer, upper, 1)
		
		' read the second pot
		potVar2 = getADC(potPin2)
		' range pot Var between 2.0 and 0
		delta = (cSng(potVar2) / 1024.0 * 2.0)
		counter = counter + delta
		sleep(0.001)
	loop
end sub

